import { EventEmitter, NgZone, OnDestroy, OnInit } from '@angular/core';
import { Observable } from 'rxjs';
import { Ref } from '../ref';
import { NgtComponentStore } from '../stores/component-store';
import { NgtStore } from '../stores/store';
import type { AnyFunction, AttachFunction, BooleanInput, NgtInstanceInternal, NgtRef, NgtUnknownInstance, UnknownRecord } from '../types';
import * as i0 from "@angular/core";
export interface NgtInstanceState<TInstance extends object = UnknownRecord> {
    instance: NgtRef<TInstance>;
    instanceArgs: unknown[];
    attach: string[] | AttachFunction;
    noAttach: boolean;
    skipParent: boolean;
    [option: string]: any;
}
export declare abstract class NgtInstance<TInstance extends object, TInstanceState extends NgtInstanceState<TInstance> = NgtInstanceState<TInstance>> extends NgtComponentStore<TInstanceState> implements OnInit, OnDestroy {
    protected zone: NgZone;
    protected store: NgtStore;
    protected parentRef: AnyFunction<NgtRef>;
    protected parentHostRef: AnyFunction<NgtRef>;
    set ref(ref: Ref);
    set skipParent(skipParent: BooleanInput);
    get skipParent(): BooleanInput;
    ready: EventEmitter<TInstance>;
    protected hasEmittedAlready: boolean;
    set noAttach(noAttach: BooleanInput);
    get noAttach(): BooleanInput;
    set attach(value: string | string[] | AttachFunction | undefined);
    get attach(): string | string[] | AttachFunction | undefined;
    readonly instance$: Observable<NgtUnknownInstance<TInstance>>;
    get instance(): NgtRef<TInstance>;
    get __ngt__(): NgtInstanceInternal;
    get parent(): NgtRef;
    protected readonly instanceArgs$: Observable<unknown[]>;
    set instanceArgs(v: unknown | unknown[]);
    get instanceArgs(): unknown | unknown[];
    constructor(zone: NgZone, store: NgtStore, parentRef: AnyFunction<NgtRef>, parentHostRef: AnyFunction<NgtRef>);
    ngOnInit(): void;
    /**
     * Sub-classes can use this function to run pre-init logic
     * @protected
     */
    protected preInit(): void;
    /**
     * Sub-classes can choose to run additional logic after init
     * @protected
     */
    protected postInit(): void;
    protected emitReady(): void;
    prepareInstance(instance: TInstance, uuid?: string): NgtUnknownInstance<TInstance>;
    /**
     * Sub-classes can use this to modify the constructor parameters
     * before calling this.prepareInstance
     */
    protected adjustCtorParams(instanceArgs: unknown[]): unknown[];
    /**
     * Sub-classes, if adjust CtorParams, can also use ctorParams$ to ensure ctor is re-invoked
     */
    protected get ctorParams$(): Observable<{}>;
    protected destroy(): void;
    /**
     * Can be used by sub-classes to run additional logic after prepare
     * @protected
     */
    protected postPrepare(_: TInstance): void;
    protected get optionFields(): Record<string, boolean>;
    private get options$();
    ngOnDestroy(): void;
    private readonly instanceReady;
    /**
     * Can be used by sub-classes to run additional logic after options are set
     * @protected
     */
    protected postSetOptions(_: TInstance): void;
    /**
     * Sub-classes can customize this to run setOptions on custom trigger
     * @protected
     */
    protected get setOptionsTrigger$(): Observable<null>;
    private readonly setOptions;
    private attachToParent;
    static ɵfac: i0.ɵɵFactoryDeclaration<NgtInstance<any, any>, [null, null, { optional: true; skipSelf: true; }, { optional: true; skipSelf: true; }]>;
    static ɵdir: i0.ɵɵDirectiveDeclaration<NgtInstance<any, any>, never, never, { "ref": "ref"; "skipParent": "skipParent"; "noAttach": "noAttach"; "attach": "attach"; }, { "ready": "ready"; }, never>;
}
